"""
This module contains demonstrations and tests of pyfdate.py
"""

from pyfdate import *
from pprint import pprint
dashes = "*"*72

def say(*args):
	"""
	In anticipation of "print" becoming a function in Python 3.
	"""
	print("".join([str(x) for x in args]))

def showTest(s):
	dashes = "_" * 70
	say("\n"*6)
	say(dashes)
	say()
	say(s)
	say(dashes)

def pause():
	print "\nPress ENTER to continue...",
	reply = raw_input()
	print

def show(s1,timeObject):
	say(s1.rstrip().ljust(55,"."),timeObject.w.ljust(10), timeObject.d)

#-----------------------------------------------------------------------
#
#-----------------------------------------------------------------------
def test_1(x):
	showTest("test_1 : moving to a specified month")

	timeObj = x.clone()
	show( "test date:",timeObj )

	timeObj = x.plus(years=1, days=1)
	show( "in a year and a day:", timeObj)

	timeObj = x.plus(months=-2)
	show( "2 months ago:", timeObj)

	timeObj = x.plus(months=1)
	show( "1 month from now:", timeObj)

	timeObj = x.plus(weeks=-1)
	show( "1 week ago:",timeObj )

	timeObj = x.plus(weeks=1)
	show( "1 week from now:",timeObj )

#-----------------------------------------------------------------------
#
#-----------------------------------------------------------------------
def test_2(x):
	showTest("test_2 : moving to a specified month")

	timeObj = x.clone()
	show( "test date:",timeObj )

	timeObj = x.gotoYearBegin()
	show( "First day of this year:", timeObj)

	timeObj = x.gotoYearEnd()
	show( "Last day of this year:", timeObj)

	timeObj = x.gotoYearBegin().plus(days=2)
	show( "Jan 3 of this year:",timeObj )

	timeObj = x.gotoYearEnd().plus(days=3)
	show( "Jan 3 of next year:",timeObj )

#-----------------------------------------------------------------------
#
#-----------------------------------------------------------------------
def test_3(x):
	showTest("test_3 : moving to a specified weekday from a weekend day")

	timeObj = Time(2007,5,13)
	show( "test date:",timeObj )

	show( "exitWeekend()", timeObj.exitWeekend())
	show( "exitWeekend(NEXT)", timeObj.exitWeekend(NEXT))
	show( "exitWeekend(PREVIOUS)", timeObj.exitWeekend(PREVIOUS))


#-----------------------------------------------------------------------
#
#-----------------------------------------------------------------------
def test_4(timeObj):
	showTest("test_4 : moving to a specified weekday")

	timeObj = Time(2007,5,14)  # a monday
	show( "test date",timeObj )
	print

	show( "gotoWeekday THURSDAY                     ", timeObj.gotoWeekday(THURSDAY))
	show( "gotoWeekday THURSDAY ignoreToday         ", timeObj.gotoWeekday(THURSDAY, useToday=False) )
	show( "gotoWeekday THURSDAY NEAREST             ", timeObj.gotoWeekday(THURSDAY, NEAREST))
	show( "gotoWeekday THURSDAY NEAREST  ignoreToday", timeObj.gotoWeekday(THURSDAY, NEAREST, useToday=False))
	show( "gotoWeekday THURSDAY PREVIOUS            ", timeObj.gotoWeekday(THURSDAY, PREVIOUS))
	show( "gotoWeekday THURSDAY PREVIOUS ignoreToday", timeObj.gotoWeekday(THURSDAY, PREVIOUS, useToday=False))
	show( "gotoWeekday THURSDAY NEXT                ", timeObj.gotoWeekday(THURSDAY, NEXT))
	show( "gotoWeekday THURSDAY NEXT     ignoreToday", timeObj.gotoWeekday(THURSDAY, NEXT, useToday=False))
	print

	show( "gotoWeekday MONDAY                     ", timeObj.gotoWeekday(MONDAY))
	show( "gotoWeekday MONDAY ignoreToday         ", timeObj.gotoWeekday(MONDAY, useToday=False) )
	show( "gotoWeekday MONDAY NEAREST             ", timeObj.gotoWeekday(MONDAY, NEAREST))
	show( "gotoWeekday MONDAY NEAREST  ignoreToday", timeObj.gotoWeekday(MONDAY, NEAREST, useToday=False))
	show( "gotoWeekday MONDAY PREVIOUS            ", timeObj.gotoWeekday(MONDAY, PREVIOUS))
	show( "gotoWeekday MONDAY PREVIOUS ignoreToday", timeObj.gotoWeekday(MONDAY, PREVIOUS, useToday=False))
	show( "gotoWeekday MONDAY NEXT                ", timeObj.gotoWeekday(MONDAY, NEXT))
	show( "gotoWeekday MONDAY NEXT     ignoreToday", timeObj.gotoWeekday(MONDAY, NEXT, useToday=False))
	print





#-----------------------------------------------------------------------
#
#-----------------------------------------------------------------------
def test_5(x):
	showTest("test_5 : moving to a specified month")

	timeObj = Time(2007,1,14)  #april
	show( "test date",timeObj )
	print

	show( "gotoMonth JULY                     ", timeObj.gotoMonth(JULY))
	show( "gotoMonth JULY ignoreToday         ", timeObj.gotoMonth(JULY,useToday=False) )
	show( "gotoMonth JULY NEAREST             ", timeObj.gotoMonth(JULY, NEAREST))
	show( "gotoMonth JULY NEAREST  ignoreToday ", timeObj.gotoMonth(JULY, NEAREST, useToday=False))
	show( "gotoMonth JULY PREVIOUS            ", timeObj.gotoMonth(JULY, PREVIOUS))
	show( "gotoMonth JULY PREVIOUS ignoreToday", timeObj.gotoMonth(JULY, PREVIOUS, useToday=False))
	show( "gotoMonth JULY NEXT                ", timeObj.gotoMonth(JULY, NEXT))
	show( "gotoMonth JULY NEXT     ignoreToday", timeObj.gotoMonth(JULY, NEXT, useToday=False))
	print

	show( "gotoMonth JANUARY                     ", timeObj.gotoMonth(JANUARY))
	show( "gotoMonth JANUARY ignoreToday         ", timeObj.gotoMonth(JANUARY,useToday=False) )
	show( "gotoMonth JANUARY NEAREST             ", timeObj.gotoMonth(JANUARY, NEAREST))
	show( "gotoMonth JANUARY NEAREST  ignoreToday ", timeObj.gotoMonth(JANUARY, NEAREST, useToday=False))
	show( "gotoMonth JANUARY PREVIOUS            ", timeObj.gotoMonth(JANUARY, PREVIOUS))
	show( "gotoMonth JANUARY PREVIOUS ignoreToday", timeObj.gotoMonth(JANUARY, PREVIOUS, useToday=False))
	show( "gotoMonth JANUARY NEXT                ", timeObj.gotoMonth(JANUARY, NEXT))
	show( "gotoMonth JANUARY NEXT     ignoreToday", timeObj.gotoMonth(JANUARY, NEXT, useToday=False))
	print


#-----------------------------------------------------------------------
#
#-----------------------------------------------------------------------
def test_6(x):
	showTest("test_6 : various output formats")
	timeObj = Time(1)
	print "The beginning of time is:"
	print timeObj.wdt
	print timeObj.get_isodatetime(" ")
	print

	timeObj = x.clone()
	show( "test date:",timeObj )

	print "It is now:"
	say( Time().wdt )   # weekday, date, time
	say( Time().dt  )   # date time
	say( Time().wd   )  # weekday date
	print

	say( Time().isodatetime)
	say( Time().get_isodatetime(' '))
	say( Time().isodate)
	say( Time().isotime)
	say( Time().unixdate)
	say( Time().get_unixdate(' '))
	print

	say( Time().civiltime)
	say( Time().civiltime2)


#-----------------------------------------------------------------------
#
#-----------------------------------------------------------------------
def test_7(x):
	showTest("test_7 : Round-trip testing")
	t1  = x.clone()

	p  = t1 - Time(1)
	t2   = Time(1) + p
	print str(t1), "==", str(p), "from the beginning of time"
	print str(t2)

#-----------------------------------------------------------------------
#
#-----------------------------------------------------------------------
def test_8(x):
	showTest("test_8 : getting a file date")

	print
	fn = "c:\\autoexec.bat"
	print "Time of", fn, "is:", Time().fromFile(fn).wdt


#-----------------------------------------------------------------------
#
#-----------------------------------------------------------------------
def test_9(x):
	showTest("test_9 : date comparisons")

	timeObj = x.clone()
	show( "test date:",timeObj )

	t1 = timeObj
	t2 = Time()
	diff = t2 - t1
	print "That was",str(diff), "ago."

	print
	if t1 >  t2: print t1.wdt, " >", t2.wdt
	if t1 == t2: print t1.wdt, "==", t2.wdt
	if t1 <  t2: print t1.wdt, "< ", t2.wdt


#-----------------------------------------------------------------------
#
#-----------------------------------------------------------------------
def test_10(x):
	showTest("test_10 : Period handling")

	print
	print "Empty period #1                 :", str(Period(datetime.timedelta(0,0)))
	print "Empty period #2                 :", Period()
	print "Period of 3 days and 45 seconds :", Period().add(days=3, seconds=45)
	print

	p = Time(2007,9,1) - Time(2007,8,1)
	print "Period between Sept 1 and Aug 1 :", p
	p = Time(2007,7,1) - Time(2007,8,1)
	print "Period between July 1 and Aug 1 :", p
	print

	p = Period(0, -1,-1,-1)
	t = Time()

	print "Now.............: ", t
	print "Negative period : ", p
	print "Now plus  period: ", t + p
	print "Now minus period: ", t - p
	print

	p = Period(0,1,1,1)
	print "Now.............: ", t
	print "Positive period : ", p
	print "Now plus  period: ", t + p
	print "Now minus period: ", t - p
	print

	t1 = t
	t2 = t - p
	print "t1      =", t1
	print "t2      =", t2
	print "t1 - t2 =", t1 - t2
	print "t2 - t1 =", t2 - t1
	print


#-----------------------------------------------------------------------
#
#-----------------------------------------------------------------------
def test_11(x):
	showTest("test_11 : Leap-year handling and cloning")

	for year in (1900, 2000, 2003, 2004):
		say("isLeapYear(", year,"): ", Time(year).isLeapYear() )
	
	say()
	t1 = Time(2004,2,29)

	t2 = t1.clone()
	say("cloned ", t1.d, " to ", t2.d)
	
	t2 = t1.clone(month=12)
	say("cloned ", t1.d, " to ", t2.d)
	
	try:
		say("\nThis should fail!")
		t2 = t1.clone(year=2005)
		say("cloned ", t1.d, " to ", t2.d)
	except ValueError, e: say(e)

		
	t1 = Time(2004,12,31)
	try:
		say("\nThis should fail!")
		t2 = t1.clone(month=11)
		say("cloned ", t1.d, " to ", t2.d)
	except ValueError, e: say(e)


	t1 = Time(2004,2,29)
	t2 = t1.goto(year=2005)
	say("moved ", t1.d, " to ", t2.d)

	t1 = Time(2004,12,31)
	t2 = t1.goto(month=11)
	say("moved ", t1.d, " to ", t2.d)


#-----------------------------------------------------------------------
#
#-----------------------------------------------------------------------
def test_12(x):
	showTest("test_12 : Some interesting calculations")
	year = 2007

	say("Here is the list of Federal holidays for ", year)

	t = Time(year,1,1)
	show("New Year's Day: January 1", 	t.exitWeekend())

	show("Martin Luther King birthday, third Monday in January",
		Time(year,1,1).gotoWeekday(MONDAY).add(weeks=2) )

	show("Washington's birthday, third Monday in February",
		Time(year,2,1).gotoWeekday(MONDAY).add(weeks=2) )

	show("Memorial Day #1, last Monday in May",
		Time(year,JUNE,1).gotoWeekday(MONDAY, PREVIOUS,useToday=False) )

	show("Memorial Day #2, last Monday in May",
		Time(year,MAY,1).gotoMonthEnd().gotoWeekday(MONDAY, PREVIOUS,useToday=True) )

	t = Time(year, JULY, 4)
	show("Independence Day, July 4", t.exitWeekend())

	show("Labor Day, first Monday in September",
		Time(year,SEPTEMBER,1).gotoWeekday(MONDAY) )

	show("Columbus Day, second Monday in October",
		Time(year,10,1).gotoWeekday(MONDAY).add(weeks=1) )

	t = Time(year,11,11)
	show("Veteran's Day, November 11", t.exitWeekend())
	say( "  * note that in 2007, Nov 11 falls on a Sunday, so it is observed on Monday.")

	show("Thanksgiving Day, fourth Thursday in November",
		Time(year,11,1).gotoWeekday(THURSDAY).add(weeks=3) )

	t = Time(year,12, 25)
	show("Christmas Day, December 25", t.exitWeekend())

	t = Time(year + 1,1,1)
	show("New Year's Day, January 1", t.exitWeekend())

#-----------------------------------------------------------------------
#
#-----------------------------------------------------------------------

if __name__ == "__main__":
	timeObj = Time(2007,4,30,18,30,45)
	print "\n"*50
	print
	print dashes
	print "Testing begins. For tests, it is now:", timeObj.wdt
	print

	if True:
		test_1(timeObj)
		pause()

	if True:
		test_2(timeObj)
		pause()

	if True:
		test_3(timeObj)
		pause()

	if True:
		test_4(timeObj)
		#pause()

	if True:
		test_5(timeObj)
		pause()


	if True:
		test_6(timeObj)
		pause()

	if True:
		test_7(timeObj)
		pause()

	if True:
		test_8(timeObj)
		pause()

	if True:
		test_9(timeObj)
		pause()

	if True:
		test_10(timeObj)
		pause()

	if True:
		test_11(timeObj)
		pause()

	if True:
		test_12(timeObj)
		pause()


	print
	print dashes
	print "Testing   ends:", Time().wdt
	print dashes